summaryrefslogtreecommitdiff
path: root/plugins/shrink-path
diff options
context:
space:
mode:
Diffstat (limited to 'plugins/shrink-path')
-rw-r--r--plugins/shrink-path/README.md88
-rw-r--r--plugins/shrink-path/shrink-path.plugin.zsh64
2 files changed, 128 insertions, 24 deletions
diff --git a/plugins/shrink-path/README.md b/plugins/shrink-path/README.md
index b990aea91..71a242b2c 100644
--- a/plugins/shrink-path/README.md
+++ b/plugins/shrink-path/README.md
@@ -1,61 +1,109 @@
-# A plugin to shrink directory paths for brevity and pretty-printing
+# shrink-path
+A plugin to shrink directory paths for brevity and pretty-printing.
+
+To use it, add `shrink-path` to the plugins array in your zshrc file:
+
+```zsh
+plugins=(... shrink-path)
+```
## Examples
For this directory tree:
```
- /home/
- me/
- foo/
- bar/
- quux/
- biz/ # The prefix b is ambiguous between bar and biz.
+/home/
+ me/
+ f o o/ # The prefix f is ambiguous between "f o o" and "f i g".
+ bar/
+ quux/
+ biz/ # The prefix b is ambiguous between bar and biz.
+ f i g/
+ baz/
```
here are the results of calling `shrink_path <option> /home/me/foo/bar/quux`:
```
- Option Result
- <none> /h/m/f/ba/q
- -l|--last /h/m/f/ba/quux
- -s|--short /h/m/f/b/q
- -t|--tilde ~/f/ba/q
- -f|--fish ~/f/b/quux
+Option Result
+<none> /h/m/f o/ba/q
+-l|--last /h/m/f o/ba/q
+-s|--short /h/m/f/b/q
+-t|--tilde ~/f o/ba/q
+-f|--fish ~/f/b/quux
+-g|--glob /h*/m*/f o*/ba*/q*
+-3 /hom/me/f o/bar/quu
+-e '$' -3 /hom$/me/f o$/bar/quu$
+-q /h/m/f\ o/ba/q
+-g -q /h*/m*/f\ o*/ba*/q*
+-x /home/me/foo/bar/quux
```
-
## Usage
For a fish-style working directory in your command prompt, add the following to
your theme or zshrc:
-```
- setopt prompt_subst
- PS1='%n@%m $(shrink_path -f)>'
+```zsh
+setopt prompt_subst
+PS1='%n@%m $(shrink_path -f)>'
```
The following options are available:
```
-f, --fish fish simulation, equivalent to -l -s -t.
+ -g, --glob Add asterisk to allow globbing of shrunk path (equivalent to -e "*")
-l, --last Print the last directory's full name.
- -s, --short Truncate directory names to the first character. Without
+ -s, --short Truncate directory names to the number of characters given by -. Without
-s, names are truncated without making them ambiguous.
-t, --tilde Substitute ~ for the home directory.
-T, --nameddirs Substitute named directories as well.
+ -# Truncate each directly to at least this many characters inclusive of the
+ ellipsis character(s) (defaulting to 1).
+ -e SYMBOL Postfix symbol(s) to indicate that a directory name had been truncated.
+ -q, --quote Quote special characters in the shrunk path
+ -x, --expand Print the full path. This takes precedence over the other options
```
The long options can also be set via zstyle, like
-```
- zstyle :prompt:shrink_path fish yes
+```zsh
+zstyle :prompt:shrink_path fish yes
```
Note: Directory names containing two or more consecutive spaces are not yet
supported.
+## Trick: toggle shrinking with a keyboard shortcut
+
+You can use the `expand` option to disable the path shrinking. You can combine that
+with a key binding widget to toggle path shrinking on and off.
+
+```zsh
+# Toggle off path shrinking
+zstyle ':prompt:shrink_path' expand true
+# Toggle on path shrinking
+zstyle -d ':prompt:shrink_path' expand
+```
+
+Combined with a widget:
+
+```zsh
+# Widget definition
+shrink-path-toggle() {
+ zstyle -t ':prompt:shrink_path' expand \
+ && zstyle -d ':prompt:shrink_path' expand \
+ || zstyle ':prompt:shrink_path' expand true
+ zle reset-prompt
+}
+zle -N shrink-path-toggle
+# Key binding to ALT+SHIFT+S
+bindkey "^[S" shrink-path-toggle
+```
+
## License
Copyright (C) 2008 by Daniel Friesel <derf@xxxxxxxxxxxxxxxxxx>
+Copyright (C) 2018-2020 by Pavel N. Krivitsky
License: WTFPL <http://www.wtfpl.net>
diff --git a/plugins/shrink-path/shrink-path.plugin.zsh b/plugins/shrink-path/shrink-path.plugin.zsh
index 86102e651..373fd5b05 100644
--- a/plugins/shrink-path/shrink-path.plugin.zsh
+++ b/plugins/shrink-path/shrink-path.plugin.zsh
@@ -9,11 +9,16 @@
# The following options are available:
#
# -f, --fish fish simulation, equivalent to -l -s -t.
+# -g, --glob Add asterisk to allow globbing of shrunk path (equivalent to -e "*")
# -l, --last Print the last directory's full name.
# -s, --short Truncate directory names to the first character. Without
# -s, names are truncated without making them ambiguous.
# -t, --tilde Substitute ~ for the home directory.
# -T, --nameddirs Substitute named directories as well.
+# -# Truncate each directly to at least this many characters inclusive of the
+# ellipsis character(s) (defaulting to 1).
+# -e SYMBOL Postfix symbol(s) to indicate that a directory name had been truncated.
+# -q, --quote Quote special characters in the shrunk path
#
# The long options can also be set via zstyle, like
# zstyle :prompt:shrink_path fish yes
@@ -37,6 +42,10 @@ shrink_path () {
typeset -i short=0
typeset -i tilde=0
typeset -i named=0
+ typeset -i length=1
+ typeset ellipsis=""
+ typeset -i quote=0
+ typeset -i expand=0
if zstyle -t ':prompt:shrink_path' fish; then
lastfull=1
@@ -50,9 +59,16 @@ shrink_path () {
zstyle -t ':prompt:shrink_path' last && lastfull=1
zstyle -t ':prompt:shrink_path' short && short=1
zstyle -t ':prompt:shrink_path' tilde && tilde=1
+ zstyle -t ':prompt:shrink_path' glob && ellipsis='*'
+ zstyle -t ':prompt:shrink_path' quote && quote=1
+ zstyle -t ':prompt:shrink_path' expand && expand=1
while [[ $1 == -* ]]; do
case $1 in
+ --)
+ shift
+ break
+ ;;
-f|--fish)
lastfull=1
short=1
@@ -61,10 +77,18 @@ shrink_path () {
-h|--help)
print 'Usage: shrink_path [-f -l -s -t] [directory]'
print ' -f, --fish fish-simulation, like -l -s -t'
+ print ' -g, --glob Add asterisk to allow globbing of shrunk path (equivalent to -e "*")'
print ' -l, --last Print the last directory''s full name'
- print ' -s, --short Truncate directory names to the first character'
+ print ' -s, --short Truncate directory names to the number of characters given by -#. Without'
+ print ' -s, names are truncated without making them ambiguous.'
print ' -t, --tilde Substitute ~ for the home directory'
print ' -T, --nameddirs Substitute named directories as well'
+ print ' -# Truncate each directly to at least this many characters inclusive of the'
+ print ' ellipsis character(s) (defaulting to 1).'
+ print ' -e SYMBOL Postfix symbol(s) to indicate that a directory name had been truncated.'
+ print ' -q, --quote Quote special characters in the shrunk path'
+ print ' -x, --expand Print the full path. This takes precedence over the other options'
+ print ''
print 'The long options can also be set via zstyle, like'
print ' zstyle :prompt:shrink_path fish yes'
return 0
@@ -76,16 +100,38 @@ shrink_path () {
tilde=1
named=1
;;
+ -[0-9]|-[0-9][0-9])
+ length=${1/-/}
+ ;;
+ -e)
+ shift
+ ellipsis="$1"
+ ;;
+ -g|--glob)
+ ellipsis='*'
+ ;;
+ -q|--quote)
+ quote=1
+ ;;
+ -x|--expand)
+ expand=1
+ ;;
esac
shift
done
+ typeset -i elllen=${#ellipsis}
typeset -a tree expn
typeset result part dir=${1-$PWD}
typeset -i i
[[ -d $dir ]] || return 0
+ if (( expand )) {
+ echo "$dir"
+ return 0
+ }
+
if (( named )) {
for part in ${(k)nameddirs}; {
[[ $dir == ${nameddirs[$part]}(/*|) ]] && dir=${dir/#${nameddirs[$part]}/\~$part}
@@ -109,12 +155,22 @@ shrink_path () {
expn=(a b)
part=''
i=0
- until [[ (( ${#expn} == 1 )) || $dir = $expn || $i -gt 99 ]] do
+ until [[ $i -gt 99 || ( $i -ge $((length - ellen)) || $dir == $part ) && ( (( ${#expn} == 1 )) || $dir = $expn ) ]]; do
(( i++ ))
part+=$dir[$i]
expn=($(echo ${part}*(-/)))
- (( short )) && break
+ (( short )) && [[ $i -ge $((length - ellen)) ]] && break
done
+
+ typeset -i dif=$(( ${#dir} - ${#part} - ellen ))
+ if [[ $dif -gt 0 ]]
+ then
+ (( quote )) && part=${(q)part}
+ part+="$ellipsis"
+ else
+ part="$dir"
+ (( quote )) && part=${(q)part}
+ fi
result+="/$part"
cd -q $dir
shift tree
@@ -9,11 +9,16 @@
# -l, --last Print the last directory's full name.
-# -s, --short Truncate directory names to the first character. Without
+# -s, --short Truncate directory names to the number of characters given by -#. Without
# -s, names are truncated without making them ambiguous.
# -t, --tilde Substitute ~ for the home directory.